@@ -13,7 +13,7 @@
  * See the main() function at the bottom that shows an app that retrives from a
  * specified url using fgets() and fread() and saves as two output files.
  *
- * Coyright (c)2003 Simtec Electronics
+ * Copyright (c) 2003 Simtec Electronics
  *
  * Re-implemented by Vincent Sanders <vince@kyllikki.org> with extensive
  * reference to original curl example code
@@ -45,7 +45,9 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <sys/time.h>
+#ifndef WIN32
+#  include <sys/time.h>
+#endif
 #include <stdlib.h>
 #include <errno.h>
 
@@ -70,7 +72,7 @@ struct fcurl_data
 typedef struct fcurl_data URL_FILE;
 
 /* exported functions */
-URL_FILE *url_fopen(char *url,const char *operation);
+URL_FILE *url_fopen(const char *url,const char *operation);
 int url_fclose(URL_FILE *file);
 int url_feof(URL_FILE *file);
 size_t url_fread(void *ptr, size_t size, size_t nmemb, URL_FILE *file);
@@ -93,11 +95,11 @@ write_callback(char *buffer,
     URL_FILE *url = (URL_FILE *)userp;
     size *= nitems;
 
-    rembuff=url->buffer_len - url->buffer_pos;//remaining space in buffer
+    rembuff=url->buffer_len - url->buffer_pos; /* remaining space in buffer */
 
     if(size > rembuff)
     {
-        //not enuf space in buffer
+        /* not enough space in buffer */
         newbuff=realloc(url->buffer,url->buffer_len + (size - rembuff));
         if(newbuff==NULL)
         {
@@ -211,7 +213,7 @@ use_buffer(URL_FILE *file,int want)
 
 
 URL_FILE *
-url_fopen(char *url,const char *operation)
+url_fopen(const char *url,const char *operation)
 {
     /* this code could check for URLs or types in the 'url' and
        basicly use the real fopen() for standard files */
@@ -219,7 +221,7 @@ url_fopen(char *url,const char *operation)
     URL_FILE *file;
     (void)operation;
 
-    file = (URL_FILE *)malloc(sizeof(URL_FILE));
+    file = malloc(sizeof(URL_FILE));
     if(!file)
         return NULL;
 
@@ -236,7 +238,7 @@ url_fopen(char *url,const char *operation)
 
         curl_easy_setopt(file->handle.curl, CURLOPT_URL, url);
         curl_easy_setopt(file->handle.curl, CURLOPT_WRITEDATA, file);
-        curl_easy_setopt(file->handle.curl, CURLOPT_VERBOSE, FALSE);
+        curl_easy_setopt(file->handle.curl, CURLOPT_VERBOSE, 0L);
         curl_easy_setopt(file->handle.curl, CURLOPT_WRITEFUNCTION, write_callback);
 
         if(!multi_handle)
@@ -466,7 +468,7 @@ main(int argc, char *argv[])
 
     int nread;
     char buffer[256];
-    char *url;
+    const char *url;
 
     if(argc < 2)
     {
@@ -481,14 +483,14 @@ main(int argc, char *argv[])
     outf=fopen("fgets.test","w+");
     if(!outf)
     {
-        perror("couldnt open fgets output file\n");
+        perror("couldn't open fgets output file\n");
         return 1;
     }
 
     handle = url_fopen(url, "r");
     if(!handle)
     {
-        printf("couldn't url_fopen()\n");
+        printf("couldn't url_fopen() %s\n", url);
         fclose(outf);
         return 2;
     }
@@ -508,13 +510,13 @@ main(int argc, char *argv[])
     outf=fopen("fread.test","w+");
     if(!outf)
     {
-        perror("couldnt open fread output file\n");
+        perror("couldn't open fread output file\n");
         return 1;
     }
 
     handle = url_fopen("testfile", "r");
     if(!handle) {
-        printf("couldn't url_fopen()\n");
+        printf("couldn't url_fopen() testfile\n");
         fclose(outf);
         return 2;
     }
@@ -533,13 +535,13 @@ main(int argc, char *argv[])
     outf=fopen("rewind.test","w+");
     if(!outf)
     {
-        perror("couldnt open fread output file\n");
+        perror("couldn't open fread output file\n");
         return 1;
     }
 
     handle = url_fopen("testfile", "r");
     if(!handle) {
-        printf("couldn't url_fopen()\n");
+        printf("couldn't url_fopen() testfile\n");
         fclose(outf);
         return 2;
     }
