@@ -5,7 +5,6 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: synctime.c,v 1.3 2006-11-19 21:55:34 bagder Exp $
  *
  * This example code only builds as-is on Windows.
  *
@@ -87,7 +86,7 @@ typedef struct
   char timeserver[MAX_STRING1];
 } conf_t;
 
-char DefaultTimeServer[4][MAX_STRING1] =
+const char DefaultTimeServer[4][MAX_STRING1] =
 {
   "http://nist.time.gov/timezone.cgi?UTC/s/0",
   "http://www.google.com/",
@@ -95,9 +94,9 @@ char DefaultTimeServer[4][MAX_STRING1] =
   "http://www.worldtime.com/cgi-bin/wt.cgi"
 };
 
-char *DayStr[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
-char *MthStr[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
-                  "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
+const char *DayStr[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
+const char *MthStr[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
+                        "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
 
 int  ShowAllHeader;
 int  AutoSyncTime;
@@ -189,7 +188,7 @@ int SyncTime_CURL_Fetch(CURL *curl, char *URL_Str, char *OutFileName,
 
   outfile = NULL;
   if (HttpGetBody == HTTP_COMMAND_HEAD)
-    curl_easy_setopt(curl, CURLOPT_NOBODY, 1);
+    curl_easy_setopt(curl, CURLOPT_NOBODY, 1L);
   else {
     outfile = fopen(OutFileName, "wb");
     curl_easy_setopt(curl, CURLOPT_WRITEDATA, outfile);
