@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2009, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,6 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: progress.c,v 1.92 2008-10-11 01:56:04 yangtse Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -211,7 +210,7 @@ void Curl_pgrsSetUploadCounter(struct SessionHandle *data, curl_off_t size)
 void Curl_pgrsSetDownloadSize(struct SessionHandle *data, curl_off_t size)
 {
   data->progress.size_dl = size;
-  if(size > 0)
+  if(size >= 0)
     data->progress.flags |= PGRS_DL_SIZE_KNOWN;
   else
     data->progress.flags &= ~PGRS_DL_SIZE_KNOWN;
@@ -220,7 +219,7 @@ void Curl_pgrsSetDownloadSize(struct SessionHandle *data, curl_off_t size)
 void Curl_pgrsSetUploadSize(struct SessionHandle *data, curl_off_t size)
 {
   data->progress.size_ul = size;
-  if(size > 0)
+  if(size >= 0)
     data->progress.flags |= PGRS_UL_SIZE_KNOWN;
   else
     data->progress.flags &= ~PGRS_UL_SIZE_KNOWN;
@@ -371,20 +370,28 @@ int Curl_pgrsUpdate(struct connectdata *conn)
 
     /* Figure out the estimated time of arrival for the upload */
     if((data->progress.flags & PGRS_UL_SIZE_KNOWN) &&
-       (data->progress.ulspeed > CURL_OFF_T_C(0)) &&
-       (data->progress.size_ul > CURL_OFF_T_C(100)) ) {
+       (data->progress.ulspeed > CURL_OFF_T_C(0))) {
       ulestimate = data->progress.size_ul / data->progress.ulspeed;
-      ulpercen = data->progress.uploaded /
-                (data->progress.size_ul/CURL_OFF_T_C(100));
+
+      if(data->progress.size_ul > CURL_OFF_T_C(10000))
+        ulpercen = data->progress.uploaded /
+          (data->progress.size_ul/CURL_OFF_T_C(100));
+      else if(data->progress.size_ul > CURL_OFF_T_C(0))
+        ulpercen = (data->progress.uploaded*100) /
+          data->progress.size_ul;
     }
 
     /* ... and the download */
     if((data->progress.flags & PGRS_DL_SIZE_KNOWN) &&
-       (data->progress.dlspeed > CURL_OFF_T_C(0)) &&
-       (data->progress.size_dl > CURL_OFF_T_C(100))) {
+       (data->progress.dlspeed > CURL_OFF_T_C(0))) {
       dlestimate = data->progress.size_dl / data->progress.dlspeed;
-      dlpercen = data->progress.downloaded /
-                (data->progress.size_dl/CURL_OFF_T_C(100));
+
+      if(data->progress.size_dl > CURL_OFF_T_C(10000))
+        dlpercen = data->progress.downloaded /
+          (data->progress.size_dl/CURL_OFF_T_C(100));
+      else if(data->progress.size_dl > CURL_OFF_T_C(0))
+        dlpercen = (data->progress.downloaded*100) /
+          data->progress.size_dl;
     }
 
     /* Now figure out which of them is slower and use that one for the
@@ -407,9 +414,11 @@ int Curl_pgrsUpdate(struct connectdata *conn)
     total_transfer = data->progress.downloaded + data->progress.uploaded;
 
     /* Get the percentage of data transfered so far */
-    if(total_expected_transfer > CURL_OFF_T_C(100))
+    if(total_expected_transfer > CURL_OFF_T_C(10000))
       total_percen = total_transfer /
-                    (total_expected_transfer/CURL_OFF_T_C(100));
+        (total_expected_transfer/CURL_OFF_T_C(100));
+    else if(total_expected_transfer > CURL_OFF_T_C(0))
+      total_percen = (total_transfer*100) / total_expected_transfer;
 
     fprintf(data->set.err,
             "\r"
