@@ -18,7 +18,6 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: share.c,v 1.28 2009-04-21 11:46:17 yangtse Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -36,7 +35,7 @@
 CURLSH *
 curl_share_init(void)
 {
-  struct Curl_share *share = calloc(sizeof(struct Curl_share), 1);
+  struct Curl_share *share = calloc(1, sizeof(struct Curl_share));
   if(share)
     share->specifier |= (1<<CURL_LOCK_DATA_SHARE);
 
