@@ -1,6 +1,10 @@
+#ifndef HEADER_CONFIG_VMS_H
+#define HEADER_CONFIG_VMS_H
+
+/*                                                                         */
 /* MSK, 02/05/04, Hand edited for trail build on Alpha V7.3, DEC C 6.5-003 */
 /* MSK, 03/09/04, Seems to work for all platforms I've built on so far.    */
-/*      Added HAVE_SYS_IOCTL_H and IOCTL_3_ARGS defines                    */
+/*      Added HAVE_SYS_IOCTL_H define                                      */
 /* TES, 10/06/04, Added MAX_INITIAL_POST_SIZE, HAVE_BASENAME               */
 /* MSK, 02/02/05, Changed HAVE_TERMIOS_H to an undef since the change in   */
 /*                getpass.c no longer undef'd it during compile.           */
@@ -9,14 +13,14 @@
 /* MSK, 01/27/07, needed to add HAVE_STRUCT_TIMEVAL define                 */
 
 /* Define cpu-machine-OS */
-#ifdef __ALPHA
-#define OS "ALPHA-HP-VMS"
+#if defined(__alpha)
+#  define OS "ALPHA-HP-VMS"
+#elif defined(__vax)
+#  define OS "VAX-HP-VMS"
+#elif defined(__ia64)
+#  define OS "IA64-HP-VMS"
 #else
-#ifdef __VAX
-#define OS "VAX-HP-VMS"
-#else
-#define OS "IA64-HP-VMS"
-#endif
+#  define OS "UNKNOWN-HP-VMS"
 #endif
 
 /* Define if you have the ANSI C header files.  */
@@ -25,15 +29,18 @@
 /* Define if you can safely include both <sys/time.h> and <time.h>.  */
 #define TIME_WITH_SYS_TIME 1
 
-/* Type to use in place of socklen_t when system does not provide it.  */
-#define socklen_t size_t
+/* The size of `int', as computed by sizeof. */
+#define SIZEOF_INT 4
 
-/* The number of bytes in a long double.  */
+/* The size of `long double', as computed by sizeof. */
 #define SIZEOF_LONG_DOUBLE 8
 
-/* The number of bytes in a long long.  */
+/* The size of `long long', as computed by sizeof. */
 #define SIZEOF_LONG_LONG 8
 
+/* The size of `short', as computed by sizeof. */
+#define SIZEOF_SHORT 2
+
 /* Define if you have the alarm function.  */
 #define HAVE_ALARM 1
 
@@ -61,6 +68,15 @@
 /* Define if you have the inet_addr function.  */
 #define HAVE_INET_ADDR 1
 
+/* Define if you have the ioctl function. */
+#define HAVE_IOCTL 1
+
+/* Define if you have a working ioctl FIONBIO function. */
+#define HAVE_IOCTL_FIONBIO 1
+
+/* Define if you have a working ioctl SIOCGIFADDR function. */
+#define HAVE_IOCTL_SIOCGIFADDR 1
+
 /* Define if you have the perror function.  */
 #define HAVE_PERROR 1
 
@@ -83,7 +99,7 @@
 #define HAVE_STRCASECMP 1
 
 /* Define if you have the strcmpi function.  */
-#define HAVE_STRCMPI 1
+/* #define HAVE_STRCMPI 1 */
 
 /* Define if you have the strdup function.  */
 #define HAVE_STRDUP 1
@@ -92,10 +108,10 @@
 #define HAVE_STRFTIME 1
 
 /* Define if you have the stricmp function.  */
-#define HAVE_STRICMP 1
+/* #define HAVE_STRICMP 1 */
 
 /* Define if you have the strstr function.  */
-#define  HAVE_STRSTR 1
+#define HAVE_STRSTR 1
 
 /* Define if you have the ftruncate function. */
 #define HAVE_FTRUNCATE 1
@@ -142,25 +158,25 @@
 #define HAVE_LIBCRYPTO 1
 
 /* Define if you have the ssl library (-lssl).  */
-#define HAVE_LIBSSL	1
+#define HAVE_LIBSSL 1
 
 /* Define if you have the <openssl/crypto.h> header file.  */
 #define HAVE_OPENSSL_CRYPTO_H 1
 
 /* Define if you have the <openssl/err.h> header file.  */
-#define HAVE_OPENSSL_ERR_H	1
+#define HAVE_OPENSSL_ERR_H 1
 
 /* Define if you have the <openssl/pem.h> header file.  */
-#define HAVE_OPENSSL_PEM_H	1
+#define HAVE_OPENSSL_PEM_H 1
 
 /* Define if you have the <openssl/rsa.h> header file.  */
 #define HAVE_OPENSSL_RSA_H 1
 
 /* Define if you have the <openssl/ssl.h> header file.  */
-#define HAVE_OPENSSL_SSL_H	1
+#define HAVE_OPENSSL_SSL_H 1
 
 /* Define if you have the <openssl/x509.h> header file.  */
-#define HAVE_OPENSSL_X509_H	1
+#define HAVE_OPENSSL_X509_H 1
 
 /*
  * This needs to be defined for OpenSSL 0.9.7 and other versions that have the
@@ -194,7 +210,7 @@
 #define HAVE_SYS_TYPES_H 1
 
 /* Define if you have the <termios.h> header file.  */
-#undef HAVE_TERMIOS_H 
+#undef HAVE_TERMIOS_H
 
 /* Define if you have the <time.h> header file.  */
 #define HAVE_TIME_H 1
@@ -212,10 +228,10 @@
 #define HAVE_GETADDRINFO 1
 
 /* Define if you have the <timeval.h> header file.  */
-#define	HAVE_TIMEVAL_H	1
+#define HAVE_TIMEVAL_H 1
 
 /* Define if you have the timeval struct.  */
-#define	HAVE_STRUCT_TIMEVAL 1
+#define HAVE_STRUCT_TIMEVAL 1
 
 /* Name of this package! */
 #define PACKAGE "not-used"
@@ -238,16 +254,17 @@
 /* Define if you have the `strtok_r' function.  */
 /* Condition lifted from <string.h>             */
 #if __CRTL_VER >= 70301000
-#define HAVE_STRTOK_R 1
+#  define HAVE_STRTOK_R 1
 #endif
 
 /* Define if you have the `strtoll' function. */
 #define HAVE_STRTOLL 1
 
 /* Define if you have the <memory.h> header file. */
-#define HAVE_MEMORY_H   1
+#define HAVE_MEMORY_H 1
 
-#define HAVE_FIONBIO	1
+/* Define to 1 if you need the memory.h header file even with stdlib.h */
+#define NEED_MEMORY_H 1
 
 /* Define if you have the `sigsetjmp' function. */
 #define HAVE_SIGSETJMP 1
@@ -261,11 +278,8 @@
 /* Define to 1 if you have the <sys/ioctl.h> header file. */
 #define HAVE_SYS_IOCTL_H 1
 
-/* IOCTL_3_ARGS defined to match the ioctl function in stropts.h */
-#define IOCTL_3_ARGS 1
-
-/* to disable LDAP */
-#define CURL_DISABLE_LDAP 1
+/* Define to 1 if you have the <stropts.h> header file. */
+#define HAVE_STROPTS_H 1
 
 /* Define if you have the getnameinfo function. */
 #define HAVE_GETNAMEINFO 1
@@ -277,7 +291,7 @@
 #define GETNAMEINFO_TYPE_ARG1 struct sockaddr *
 
 /* Define to the type of arg 2 for getnameinfo. */
-#define GETNAMEINFO_TYPE_ARG2 socklen_t
+#define GETNAMEINFO_TYPE_ARG2 size_t
 
 /* Define to the type of args 4 and 6 for getnameinfo. */
 #define GETNAMEINFO_TYPE_ARG46 size_t
@@ -354,3 +368,4 @@
 /* Define to hide dollar sign from compilers in strict ansi mode. */
 #define decc_translate_vms(__s) decc$translate_vms(__s)
 
+#endif /* HEADER_CONFIG_VMS_H */
